@@ -8,7 +8,7 @@ pub mod timer;
 pub mod ws;
 
 pub trait AsyncWrapper<U, S> {
-    fn new(sync: S) -> Self;
+    fn new(unblocker: U, sync: S) -> Self;
 }
 
 pub trait Asyncify {
@@ -18,25 +18,25 @@ pub trait Asyncify {
     where
         Self: Sized,
     {
-        Self::AsyncWrapper::new(self)
+        Self::AsyncWrapper::new((), self)
     }
 
     fn as_async(&mut self) -> Self::AsyncWrapper<&mut Self> {
-        Self::AsyncWrapper::new(self)
+        Self::AsyncWrapper::new((), self)
     }
 }
 
 pub trait UnblockingAsyncify {
     type AsyncWrapper<U, S>: AsyncWrapper<U, S>;
 
-    fn into_async_with_unblocker<U>(self) -> Self::AsyncWrapper<U, Self>
+    fn into_async_with_unblocker<U>(self, unblocker: U) -> Self::AsyncWrapper<U, Self>
     where
         Self: Sized,
     {
-        Self::AsyncWrapper::new(self)
+        Self::AsyncWrapper::new(unblocker, self)
     }
 
-    fn as_async_with_unblocker<U>(&mut self) -> Self::AsyncWrapper<U, &mut Self> {
-        Self::AsyncWrapper::new(self)
+    fn as_async_with_unblocker<U>(&mut self, unblocker: U) -> Self::AsyncWrapper<U, &mut Self> {
+        Self::AsyncWrapper::new(unblocker, self)
     }
 }
